@@ -4,10 +4,11 @@ from copy import copy
 from decimal import Decimal, InvalidOperation
 from distutils.version import LooseVersion
 from math import isclose, isfinite
+from typing import Type
 from zlib import compress
 
 import pytest
-from conftest import skip_if_pypy
+from conftest import needs_libqpdf_v, skip_if_pypy
 from hypothesis import assume, example, given
 from hypothesis.strategies import (
     binary,
@@ -32,6 +33,7 @@ from pikepdf import (
     String,
 )
 from pikepdf import _qpdf as qpdf
+from pikepdf.models import parse_content_stream
 
 # pylint: disable=eval-used, redefined-outer-name
 
@@ -158,6 +160,10 @@ def test_list_apis():
     assert a == pikepdf.Array([1, Name.Foo, 4])
     a.extend([42, 666])
     assert a == pikepdf.Array([1, Name.Foo, 4, 42, 666])
+    with pytest.raises(ValueError, match='object is not a dictionary'):
+        del a.ImaginaryKey
+    with pytest.raises(TypeError, match=r"items\(\) not available"):
+        a.items()
 
 
 @skip_if_pypy
@@ -203,44 +209,45 @@ def test_wrap_array():
     assert Array([42]).wrap_in_array() == Array([42])
 
 
-def test_name_equality():
-    # Who needs transitivity? :P
-    # While this is less than ideal ('/Foo' != b'/Foo') it allows for slightly
-    # sloppy tests like if colorspace == '/Indexed' without requiring
-    # Name('/Indexed') everywhere
-    assert Name('/Foo') == '/Foo'
-    assert Name('/Foo') == b'/Foo'
-    assert Name.Foo == Name('/Foo')
-
-
 def test_no_len():
     with pytest.raises(TypeError):
         len(Name.Foo)
         len(String('abc'))
 
 
-def test_unslashed_name():
-    with pytest.raises(ValueError, match='must begin with'):
-        Name('Monty') not in []  # pylint: disable=expression-not-assigned
+class TestName:
+    def test_name_equality(self):
+        # Who needs transitivity? :P
+        # While this is less than ideal ('/Foo' != b'/Foo') it allows for slightly
+        # sloppy tests like if colorspace == '/Indexed' without requiring
+        # Name('/Indexed') everywhere
+        assert Name('/Foo') == '/Foo'
+        assert Name('/Foo') == b'/Foo'
+        assert Name.Foo == Name('/Foo')
 
+    def test_unslashed_name(self):
+        with pytest.raises(ValueError, match='must begin with'):
+            Name('Monty') not in []  # pylint: disable=expression-not-assigned
 
-def test_empty_name():
-    with pytest.raises(ValueError):
-        Name('')
-    with pytest.raises(ValueError):
-        Name('/')
+    def test_empty_name(self):
+        with pytest.raises(ValueError):
+            Name('')
+        with pytest.raises(ValueError):
+            Name('/')
 
+    def test_forbidden_name_usage(self):
+        with pytest.raises(AttributeError, match="may not be set on pikepdf.Name"):
+            Name.Monty = Name.Python
+        with pytest.raises(TypeError, match="not subscriptable"):
+            Name['/Monty']  # pylint: disable=pointless-statement
+        if sys.implementation.name == 'pypy':
+            pytest.xfail(reason="pypy seems to do setattr differently")
+        with pytest.raises(AttributeError, match="has no attribute"):
+            monty = Name.Monty
+            monty.Attribute = 42
 
-def test_forbidden_name_usage():
-    with pytest.raises(AttributeError, match="may not be set on pikepdf.Name"):
-        Name.Monty = Name.Python
-    with pytest.raises(TypeError, match="not subscriptable"):
-        Name['/Monty']  # pylint: disable=pointless-statement
-    if sys.implementation.name == 'pypy':
-        pytest.xfail(reason="pypy seems to do setattr differently")
-    with pytest.raises(AttributeError, match="has no attribute"):
-        monty = Name.Monty
-        monty.Attribute = 42
+    def test_bytes_of_name(self):
+        assert bytes(Name.ABC) == b'/ABC'
 
 
 class TestHashViolation:
@@ -346,6 +353,14 @@ class TestRepr:
             assert '.get_object' in repr(pdf.Root.Circular)
 
 
+def test_operator_inline(resources):
+    with pikepdf.open(resources / 'image-mono-inline.pdf') as pdf:
+        instructions = parse_content_stream(pdf.pages[0], operators='BI ID EI')
+        assert len(instructions) == 1
+        operands, operator = instructions[0]
+        assert operator == pikepdf.Operator("INLINE IMAGE")
+
+
 def test_utf16_error():
     with pytest.raises((UnicodeEncodeError, RuntimeError)):
         str(encode('\ud801'))
@@ -406,9 +421,34 @@ class TestDictionary:
         with pytest.raises(TypeError):
             d.page_contents_add(b'', True)
 
-    def test_bad_name(self):
-        with pytest.raises(ValueError, match=r"must begin with '/'"):
+    def test_bad_name_init(self):
+        with pytest.raises(KeyError, match=r"must begin with '/'"):
             pikepdf.Dictionary({'/Slash': 'dot', 'unslash': 'error'})
+        with pytest.raises(KeyError, match=r"must begin with '/'"):
+            pikepdf.Dictionary({'/': 'slash'})
+
+    def test_bad_name_set(self):
+        d = pikepdf.Dictionary()
+        d['/Slash'] = 'dot'
+        with pytest.raises(KeyError, match=r"must begin with '/'"):
+            d['unslash'] = 'error'
+        with pytest.raises(KeyError, match=r"may not be '/'"):
+            d['/'] = 'error'
+
+    def test_del_missing_key(self):
+        d = pikepdf.Dictionary(A='a')
+        with pytest.raises(KeyError):
+            del d.B
+
+    def test_int_access(self):
+        d = pikepdf.Dictionary()
+        with pytest.raises(TypeError, match="not an array"):
+            d[0] = 3
+
+    def test_wrong_contains_type(self):
+        d = pikepdf.Dictionary()
+        with pytest.raises(TypeError, match="can only contain Names"):
+            pikepdf.Array([3]) in d
 
 
 def test_not_convertible():
@@ -553,6 +593,11 @@ def test_object_iteration(sandwich):
     assert expected == loops
 
 
+def test_object_not_iterable():
+    with pytest.raises(TypeError, match="__iter__ not available"):
+        iter(pikepdf.Name.A)
+
+
 @pytest.mark.parametrize(
     'obj', [Array([1]), Dictionary({'/A': 'b'}), Operator('q'), String('s')]
 )
@@ -575,10 +620,17 @@ def test_object_classes():
         assert issubclass(cls, Object)
 
 
-def test_operator_create():
-    Operator('q')
-    assert Operator('q') == Operator('q')
-    assert Operator('q') != Operator('Q')
+class TestOperator:
+    def test_operator_create(self):
+        Operator('q')
+        assert Operator('q') == Operator('q')
+        assert Operator('q') != Operator('Q')
+
+    def test_operator_str(self):
+        assert str(Operator('Do')) == 'Do'
+
+    def test_operator_bytes(self):
+        assert bytes(Operator('cm')) == b'cm'
 
 
 @pytest.fixture(scope="function")
@@ -676,3 +728,19 @@ def test_object_mapping(sandwich):
     assert '42' not in object_mapping
     assert '/R12' in object_mapping
     assert '/R12' in object_mapping.keys()
+
+
+@needs_libqpdf_v('10.3.0', reason="behavior of replace improved in v10.3.0")
+def test_replace_object(sandwich):
+    d = Dictionary(Type=Name.Dummy)
+    sandwich._replace_object(sandwich.pages[0].objgen, d)
+    assert sandwich.pages[0] == d
+
+
+@needs_libqpdf_v('10.3.0', reason="behavior of swap improved in v10.3.0")
+def test_swap_object(resources):
+    with Pdf.open(resources / 'fourpages.pdf') as pdf:
+        pdf.pages[0].MarkPage0 = True
+        pdf._swap_objects(pdf.pages[0].objgen, pdf.pages[1].objgen)
+        assert pdf.pages[1].MarkPage0
+        assert Name.MarkPage0 not in pdf.pages[0]
